import secrets
import subprocess
import uvicorn
from fastapi import FastAPI, status, Depends, HTTPException
from fastapi.middleware.cors import CORSMiddleware
from fastapi.security import HTTPBasic, HTTPBasicCredentials
from typing import Annotated


app = FastAPI()

security = HTTPBasic()

app.add_middleware(
    CORSMiddleware,
    allow_origins=['*'],
    allow_methods=["*"],
    allow_headers=["*"]
)   

def execCommand(cmd: str):
    result = subprocess.run(cmd, stdout=subprocess.PIPE, shell=True)
    if(result.stdout == ''):
        return result.stderr
    else:
        return result.stdout

def authorize(
    credentials: Annotated[HTTPBasicCredentials, Depends(security)]
):
    current_username_bytes = credentials.username.encode("utf8")
    correct_username_bytes = b"kiosk"
    is_correct_username = secrets.compare_digest(
        current_username_bytes, correct_username_bytes
    )
    current_password_bytes = credentials.password.encode("utf8")
    correct_password_bytes = b"kenny"
    is_correct_password = secrets.compare_digest(
        current_password_bytes, correct_password_bytes
    )
    if not (is_correct_username and is_correct_password):
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="Incorrect username or password",
            headers={"WWW-Authenticate": "Basic"},
        )
    
def getTwingateContainerName(shell_output: str):
    bytes_str = shell_output.decode("utf-8")
    str_index = bytes_str.rfind('twingate')
    return bytes_str[str_index:].strip()

def getUpdateStatus(pkg_name: str):
    shell_output_upgrade = execCommand('apt list --upgradeable {}'.format(pkg_name))
    shell_output_current = execCommand('apt list {}'.format(pkg_name))
    upgradable = getUpgradbleVersion(shell_output_upgrade)
    current = getCurrentInstalledVersion(shell_output_current)
    if(upgradable == ''):
        upgradable = current
    return {
        'current': current,
        'newest': upgradable
    }

def getUpgradbleVersion(shell_output: str):
    # sudo apt list --upgradeable nss-core 
    # no new version available resp. package not installed:
    # b'Auflistung\xe2\x80\xa6\n'
    example = shell_output.decode('utf-8')
    # new version available:
    # b'Auflistung\xe2\x80\xa6\nnss-core/unbekannt 2.1.0 amd64 [aktualisierbar von: 2.0.1]\n'
    # example = b'Auflistung\xe2\x80\xa6\nnss-core/unbekannt 2.1.0 amd64 [aktualisierbar von: 2.0.1]\n'.decode("utf-8").decode();
    if(len(example.split(' ')) <= 1):
        # package not upgradble
        new_version_version = ''
    else:
        new_version_version = example.split(' ')[1]
    return new_version_version
    
def getCurrentInstalledVersion(shell_output: str):
    # sudo apt list nss-core 
    example = shell_output.decode('utf-8')
    if(len(example.split(' ')) <= 1):
        # package not installed
        current_version = 'NOT INSTALLED'
    elif(len(example.split(' ')) > 5):
        # there is an update available, so take the version of the end of the string
        current_version = example.split(' ')[-1].split(']')[0]
    else:
        current_version = example.split(' ')[1]
    return current_version


# GET
@app.get("/twingate", status_code=status.HTTP_200_OK, dependencies=[Depends(authorize)])
async def twingateStatus():
    shell_output = execCommand('docker ps | grep twingate')
    # example: b'1155ec5f151b   twingate/connector:1   "/connectord"            3 months ago   Up 21 seconds (healthy)                                                 twingate-beneficial-cassowary\n'
    # or: b''
    return getTwingateContainerName(shell_output=shell_output)

@app.get("/update/status", status_code=status.HTTP_200_OK, dependencies=[Depends(authorize)])
async def updateStatus():
    # get nss-core status
    nss_core_json = getUpdateStatus('nss-core')
    # get nss-frontend status
    nss_frontend_json = getUpdateStatus('nss-frontend')
    # get nss-core status
    nss_shell_json = getUpdateStatus('nss-shell-executor')
    return {
        'core': nss_core_json,
        'frontend': nss_frontend_json,
        'shellExecutor': nss_shell_json
    }

# PUT
@app.put("/twingate/start", status_code=status.HTTP_200_OK, dependencies=[Depends(authorize)])
async def twingateStart():
    shell_output = execCommand('docker ps -a | grep twingate')
    conatiner_name = getTwingateContainerName(shell_output=shell_output)
    return execCommand('docker start {0}'.format(conatiner_name))

@app.put("/update/execute", status_code=status.HTTP_200_OK, dependencies=[Depends(authorize)])
async def updateExecute():
    ret = execCommand('sudo -S /usr/bin/apt install nss-core')
    ret += execCommand('sudo -S /usr/bin/apt install nss-frontend')
    ret += execCommand('sudo -S /usr/bin/apt install nss-shell-executor')
    ret += execCommand('sudo -S /sbin/reboot')
    return ret

@app.put("/reboot", status_code=status.HTTP_200_OK, dependencies=[Depends(authorize)])
async def reboot():
    return execCommand('sudo -S /sbin/reboot')

@app.put("/twingate/stop", status_code=status.HTTP_200_OK, dependencies=[Depends(authorize)])
async def twingateStop():
    shell_output = execCommand('docker ps -a | grep twingate')
    conatiner_name = getTwingateContainerName(shell_output=shell_output)
    return execCommand('docker stop {0}'.format(conatiner_name))

if __name__ == "__main__":
    uvicorn.run(app, host="0.0.0.0", port=8081)